function C = quad3dcovpoly(sx, sy, sz, x, y, z)
% Theoretical noise covariance structure corresponding to a 3D quadratic function.
%
% Input arguments:
% sx, sy, sz:
%    vector of noise covariance components (up to scale)
% x, y, z:
%    vector of observed data

% Copyright 2008-2009 Levente Hunyadi

validateattributes(sx, {'numeric'}, {'nonnegative','scalar'});
validateattributes(sy, {'numeric'}, {'nonnegative','scalar'});
validateattributes(sz, {'numeric'}, {'nonnegative','scalar'});
n = sqrt(sx.^2 + sy.^2 + sz.^2);
sx = sx / n;
sy = sy / n;
sz = sz / n;

validateattributes(x, {'numeric'}, {'real','nonempty','vector'});
validateattributes(y, {'numeric'}, {'real','nonempty','vector'});
validateattributes(z, {'numeric'}, {'real','nonempty','vector'});
assert(numel(x) == numel(y) && numel(y) == numel(z), ...
    'quad3dcov:DimensionMismatch', 'Data vectors x, y and z must have the same number of elements.');
x = x(:);
y = y(:);
z = z(:);

% terms are ordered as [x^2 y^2 z^2 x*y x*z y*z x y z 1]
C = zeros(10,10,3);
C(:,:,1) = ...
[ mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0), mean(0) ...
];
C(:,:,2) = ...
[            mean(6.*sx.^2.*x.^2), mean(sx.^2.*y.^2 + sy.^2.*x.^2), mean(sx.^2.*z.^2 + sz.^2.*x.^2),            mean(3.*sx.^2.*x.*y),            mean(3.*sx.^2.*x.*z),               mean(sx.^2.*y.*z), mean(3.*sx.^2.*x),    mean(sx.^2.*y),    mean(sx.^2.*z), mean(sx.^2) ...
; mean(sx.^2.*y.^2 + sy.^2.*x.^2),            mean(6.*sy.^2.*y.^2), mean(sy.^2.*z.^2 + sz.^2.*y.^2),            mean(3.*sy.^2.*x.*y),               mean(sy.^2.*x.*z),            mean(3.*sy.^2.*y.*z),    mean(sy.^2.*x), mean(3.*sy.^2.*y),    mean(sy.^2.*z), mean(sy.^2) ...
; mean(sx.^2.*z.^2 + sz.^2.*x.^2), mean(sy.^2.*z.^2 + sz.^2.*y.^2),            mean(6.*sz.^2.*z.^2),               mean(sz.^2.*x.*y),            mean(3.*sz.^2.*x.*z),            mean(3.*sz.^2.*y.*z),    mean(sz.^2.*x),    mean(sz.^2.*y), mean(3.*sz.^2.*z), mean(sz.^2) ...
;            mean(3.*sx.^2.*x.*y),            mean(3.*sy.^2.*x.*y),               mean(sz.^2.*x.*y), mean(sx.^2.*y.^2 + sy.^2.*x.^2),               mean(sx.^2.*y.*z),               mean(sy.^2.*x.*z),    mean(sx.^2.*y),    mean(sy.^2.*x),           mean(0),     mean(0) ...
;            mean(3.*sx.^2.*x.*z),               mean(sy.^2.*x.*z),            mean(3.*sz.^2.*x.*z),               mean(sx.^2.*y.*z), mean(sx.^2.*z.^2 + sz.^2.*x.^2),               mean(sz.^2.*x.*y),    mean(sx.^2.*z),           mean(0),    mean(sz.^2.*x),     mean(0) ...
;               mean(sx.^2.*y.*z),            mean(3.*sy.^2.*y.*z),            mean(3.*sz.^2.*y.*z),               mean(sy.^2.*x.*z),               mean(sz.^2.*x.*y), mean(sy.^2.*z.^2 + sz.^2.*y.^2),           mean(0),    mean(sy.^2.*z),    mean(sz.^2.*y),     mean(0) ...
;               mean(3.*sx.^2.*x),                  mean(sy.^2.*x),                  mean(sz.^2.*x),                  mean(sx.^2.*y),                  mean(sx.^2.*z),                         mean(0),       mean(sx.^2),           mean(0),           mean(0),     mean(0) ...
;                  mean(sx.^2.*y),               mean(3.*sy.^2.*y),                  mean(sz.^2.*y),                  mean(sy.^2.*x),                         mean(0),                  mean(sy.^2.*z),           mean(0),       mean(sy.^2),           mean(0),     mean(0) ...
;                  mean(sx.^2.*z),                  mean(sy.^2.*z),               mean(3.*sz.^2.*z),                         mean(0),                  mean(sz.^2.*x),                  mean(sz.^2.*y),           mean(0),           mean(0),       mean(sz.^2),     mean(0) ...
;                     mean(sx.^2),                     mean(sy.^2),                     mean(sz.^2),                         mean(0),                         mean(0),                         mean(0),           mean(0),           mean(0),           mean(0),     mean(0) ...
];
C(:,:,3) = ...
[     mean(-3.*sx.^4), mean(-sx.^2.*sy.^2), mean(-sx.^2.*sz.^2),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(-sx.^2.*sy.^2),     mean(-3.*sy.^4), mean(-sy.^2.*sz.^2),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
; mean(-sx.^2.*sz.^2), mean(-sy.^2.*sz.^2),     mean(-3.*sz.^4),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0), mean(-sx.^2.*sy.^2),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0), mean(-sx.^2.*sz.^2),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0),             mean(0), mean(-sy.^2.*sz.^2), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
;             mean(0),             mean(0),             mean(0),             mean(0),             mean(0),             mean(0), mean(0), mean(0), mean(0), mean(0) ...
];